=== modified file 'drizzled/module/registry.cc'
--- drizzled/module/registry.cc	2010-08-12 17:36:08 +0000
+++ drizzled/module/registry.cc	2010-09-22 13:04:27 +0000
@@ -38,7 +38,21 @@
 
 module::Registry::~Registry()
 {
-  map<string, plugin::Plugin *>::iterator plugin_iter= plugin_registry.begin();
+  map<string, plugin::Plugin *>::iterator plugin_iter;
+
+  /* Give all plugins a chance to cleanup, before
+   * all plugins are deleted.
+   * This can be used if shutdown code references
+   * other plugins.
+   */
+  plugin_iter= plugin_registry.begin();
+  while (plugin_iter != plugin_registry.end())
+  {
+    (*plugin_iter).second->shutdownPlugin();
+    ++plugin_iter;
+  }
+
+  plugin_iter= plugin_registry.begin();
   while (plugin_iter != plugin_registry.end())
   {
     delete (*plugin_iter).second;

=== modified file 'drizzled/plugin/plugin.h'
--- drizzled/plugin/plugin.h	2010-05-15 18:23:34 +0000
+++ drizzled/plugin/plugin.h	2010-09-22 13:04:27 +0000
@@ -49,6 +49,15 @@
   explicit Plugin(std::string in_name, std::string in_type_name);
   virtual ~Plugin() {}
 
+  /*
+   * This method is called for all plug-ins on shutdown,
+   * _before_ the plug-ins are deleted. It can be used
+   * when shutdown code references other plug-ins.
+   */
+  virtual void shutdownPlugin()
+  {
+  }
+ 
   void activate()
   {
     is_active= true;

=== modified file 'plugin/pbxt/src/ha_pbxt.cc'
--- plugin/pbxt/src/ha_pbxt.cc	2010-08-08 01:18:02 +0000
+++ plugin/pbxt/src/ha_pbxt.cc	2010-09-22 13:04:27 +0000
@@ -1311,10 +1311,9 @@
 	XT_RETURN(1);
 }
 
-static int pbxt_end(void *)
+void PBXTStorageEngine::shutdownPlugin()
 {
-	XTThreadPtr		self;
-	int				err = 0;
+	XTThreadPtr self;
 
 	XT_TRACE_CALL();
 
@@ -1329,13 +1328,11 @@
 			ha_exit(self);
 		}
 	}
-
-	XT_RETURN(err);
 }
 
 PBXTStorageEngine::~PBXTStorageEngine()
 {
-  pbxt_end(NULL);
+	/* We do nothing here, because it is now all done in shutdownPlugin(). */
 }
 
 #ifndef DRIZZLED
@@ -5734,9 +5731,16 @@
 	if (!self->st_database)
 		xt_ha_open_database_of_table(self, NULL);
 
-	if (!xt_xn_begin(self)) {
+	/* startTransaction() calls registerResourceForTransaction() calls engine->startTransaction(), and then
+	 * startTransaction() calls doStartTransaction()
+	 * Which leads to this function being called twice!?
+	 * So added the self->st_xact_data test below.
+	 */
+	if (!self->st_xact_data) {
+		if (!xt_xn_begin(self)) {
 			err = xt_ha_pbxt_thread_error_for_mysql(thd, self, /*pb_ignore_dup_key*/false);
 			//pb_ex_in_use = 0;
+		}
 	}
 
 	return err;

=== modified file 'plugin/pbxt/src/ha_pbxt.h'
--- plugin/pbxt/src/ha_pbxt.h	2010-09-13 22:31:32 +0000
+++ plugin/pbxt/src/ha_pbxt.h	2010-09-22 13:04:27 +0000
@@ -104,7 +104,9 @@
 
 	/* override */ bool doDoesTableExist(Session&, const TableIdentifier &identifier);
 
-        ~PBXTStorageEngine();
+	virtual void shutdownPlugin();
+
+	~PBXTStorageEngine();
 };
 
 typedef PBXTStorageEngine handlerton;

=== modified file 'plugin/pbxt/src/myxt_xt.cc'
--- plugin/pbxt/src/myxt_xt.cc	2010-08-26 15:59:31 +0000
+++ plugin/pbxt/src/myxt_xt.cc	2010-09-22 13:04:27 +0000
@@ -28,7 +28,11 @@
 #include "xt_defs.h"
 
 #ifdef DRIZZLED
+#include <drizzled/internal/my_pthread.h>
 #include <drizzled/plugin.h>
+#include <drizzled/plugin/client.h>
+#include <drizzled/plugin/null_client.h>
+#include <drizzled/plugin/listen.h>
 #include <drizzled/show.h>
 #include <drizzled/data_home.h>
 #include <drizzled/field/blob.h>
@@ -3288,7 +3292,20 @@
 xtPublic void *myxt_create_thread()
 {
 #ifdef DRIZZLED
-	return (void *) 1;
+	Client		*client;
+	Session		*session;
+
+	if (drizzled::internal::my_thread_init()) {
+		xt_register_error(XT_REG_CONTEXT, XT_ERR_MYSQL_ERROR, 0, "Unable to initialize MySQL threading");
+		return NULL;
+	}
+
+	if (!(client = new NullClient()))
+		return NULL;
+	session = new Session(client);
+	session->thread_stack = (char *) &session;
+	session->storeGlobals();
+	return (void *) session;
 #else
 	THD *new_thd;
 
@@ -3366,12 +3383,15 @@
 }
 
 #ifdef DRIZZLED
-xtPublic void myxt_destroy_thread(void *, xtBool)
-{
-}
-
-xtPublic void myxt_delete_remaining_thread()
-{
+xtPublic void myxt_destroy_thread(void *s, xtBool end_threads)
+{
+	Session *session = (Session *) s;
+
+	session->lockForDelete();
+	delete session;
+
+	if (end_threads)
+		drizzled::internal::my_thread_end();
 }
 #else
 xtPublic void myxt_destroy_thread(void *thread, xtBool end_threads)
@@ -3400,6 +3420,7 @@
 	if (end_threads)
 		my_thread_end();
 }
+#endif
 
 xtPublic void myxt_delete_remaining_thread()
 {
@@ -3408,7 +3429,6 @@
 	if ((thd = current_thd))
 		myxt_destroy_thread((void *) thd, TRUE);
 }
-#endif
 
 xtPublic XTThreadPtr myxt_get_self()
 {

